#include "Minimax.h"
#include "ChessGame.h"

using namespace chessy;


template<Side side>
float chessy::MiniMax::evaluate(void) const
{
    return 0.0f;
}


template<Side side>
float MiniMax::minimax(int depth)
{
    if (depth == 0) {
        return { {0, 0}, evaluate() };
    }
    BestMove bestMove = { {0, 0}, 0.0f };

    Bitboard friends;
    Bitboard enemies;
    Board& board = game.getBoard();
    if (side == WHITE_SIDE) {
        friends = board.getWhites();
        enemies = board.getBlacks();
    }
    else {
        friends = board.getBlacks();
        enemies = board.getWhites();
    }

    PawnPushGenerator<side> mg{ game };
    for (Move push : mg) {
        Bitboard& pawns = board.getPawns<side>();
        Bitboard temp = pawns;
        pawns &= ~(Bitboard::fromIndex(push.origin));
        pawns |= Bitboard::fromIndex(push.destination);
        bestMove.overwriteIfBetter(minmax<!side>(depth - 1));
    }
        
    PositionSet knights = game.getBoard().getBlackKnights();
    for (auto knight : knights) {
        for (auto pos : KnightMoveGenerator{ knight, blacks }) {
            ret.push_back(Move{ knight, pos });
        }
    }

    PositionSet bishops = cg.getBoard().getBlackBishops();
    for (auto bishop : bishops) {
        for (auto pos : PrimitiveBishopMoveGenerator{ bishop, whites, blacks }) {
            ret.push_back(Move{ bishop, pos });
        }
    }

    PositionSet rooks = cg.getBoard().getBlackRooks();
    for (auto rook : rooks) {
        for (auto pos : PrimitiveRookMoveGenerator{ rook, whites, blacks }) {
            //cout << "rook: " << Move(rook, pos).asString() << endl;
        }
    }

    PositionSet queens = cg.getBoard().getBlackQueens();
    for (auto queen : queens) {
        for (auto pos : PrimitiveQueenMoveGenerator{ queen, whites, blacks }) {
            //cout << "queen: " << Move(queen, pos).asString() << endl;
        }
    }

    Bitboard king = cg.getBoard().getBlackKings();
    Index kingIndex = king.getLeastSignificantBit();
    for (auto pos : KingMoveGenerator{ king, blacks }) {
        //cout << "king: " << Move(kingIndex, pos).asString() << endl;
    }
    return {0, 0};
}